@@ -14,7 +14,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import logging
 import os.path
 import tornado.auth
 import tornado.escape
@@ -22,7 +21,6 @@ import tornado.httpserver
 import tornado.ioloop
 import tornado.options
 import tornado.web
-import uimodules
 
 from tornado.options import define, options
 
@@ -48,8 +46,9 @@ class Application(tornado.web.Application):
             xsrf_cookies=True,
             facebook_api_key=options.facebook_api_key,
             facebook_secret=options.facebook_secret,
-            ui_modules= {"Post": PostModule},
+            ui_modules={"Post": PostModule},
             debug=True,
+            autoescape=None,
         )
         tornado.web.Application.__init__(self, handlers, **settings)
 
@@ -61,32 +60,38 @@ class BaseHandler(tornado.web.RequestHandler):
         return tornado.escape.json_decode(user_json)
 
 
-class MainHandler(BaseHandler, tornado.auth.FacebookMixin):
+class MainHandler(BaseHandler, tornado.auth.FacebookGraphMixin):
     @tornado.web.authenticated
     @tornado.web.asynchronous
     def get(self):
-        self.facebook_request(
-            method="stream.get",
-            callback=self.async_callback(self._on_stream),
-            session_key=self.current_user["session_key"])
+        self.facebook_request("/me/home", self._on_stream,
+                              access_token=self.current_user["access_token"])
 
     def _on_stream(self, stream):
         if stream is None:
             # Session may have expired
             self.redirect("/auth/login")
             return
-        # Turn profiles into a dict mapping id => profile
-        stream["profiles"] = dict((p["id"], p) for p in stream["profiles"])
         self.render("stream.html", stream=stream)
 
 
-class AuthLoginHandler(BaseHandler, tornado.auth.FacebookMixin):
+class AuthLoginHandler(BaseHandler, tornado.auth.FacebookGraphMixin):
     @tornado.web.asynchronous
     def get(self):
-        if self.get_argument("session", None):
-            self.get_authenticated_user(self.async_callback(self._on_auth))
+        my_url = (self.request.protocol + "://" + self.request.host +
+                  "/auth/login?next=" +
+                  tornado.escape.url_escape(self.get_argument("next", "/")))
+        if self.get_argument("code", False):
+            self.get_authenticated_user(
+                redirect_uri=my_url,
+                client_id=self.settings["facebook_api_key"],
+                client_secret=self.settings["facebook_secret"],
+                code=self.get_argument("code"),
+                callback=self._on_auth)
             return
-        self.authorize_redirect("read_stream")
+        self.authorize_redirect(redirect_uri=my_url,
+                                client_id=self.settings["facebook_api_key"],
+                                extra_params={"scope": "read_stream"})
     
     def _on_auth(self, user):
         if not user:
@@ -95,25 +100,15 @@ class AuthLoginHandler(BaseHandler, tornado.auth.FacebookMixin):
         self.redirect(self.get_argument("next", "/"))
 
 
-class AuthLogoutHandler(BaseHandler, tornado.auth.FacebookMixin):
-    @tornado.web.asynchronous
+class AuthLogoutHandler(BaseHandler, tornado.auth.FacebookGraphMixin):
     def get(self):
         self.clear_cookie("user")
-        if not self.current_user:
-            self.redirect(self.get_argument("next", "/"))
-            return
-        self.facebook_request(
-            method="auth.revokeAuthorization",
-            callback=self.async_callback(self._on_deauthorize),
-            session_key=self.current_user["session_key"])
-
-    def _on_deauthorize(self, response):
         self.redirect(self.get_argument("next", "/"))
 
 
 class PostModule(tornado.web.UIModule):
-    def render(self, post, actor):
-        return self.render_string("modules/post.html", post=post, actor=actor)
+    def render(self, post):
+        return self.render_string("modules/post.html", post=post)
 
 
 def main():
